package com.unis.transaction.server.config;

import com.unis.transaction.common.constant.BacklogConstant;
import lombok.extern.slf4j.Slf4j;
import org.springframework.amqp.core.Binding;
import org.springframework.amqp.core.BindingBuilder;
import org.springframework.amqp.core.Queue;
import org.springframework.amqp.core.TopicExchange;
import org.springframework.amqp.rabbit.connection.CachingConnectionFactory;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.amqp.support.converter.Jackson2JsonMessageConverter;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

/**
 * 
 * @Description：待办公共配置
 * @author：王帅 
 * @date: 2020年2月27日
 */
@Configuration
@Slf4j
public class RabbitConfig {

	@Autowired
	private CachingConnectionFactory connectionFactory;

	@Bean
	public RabbitTemplate rabbitTemplate() {
		RabbitTemplate rabbitTemplate = new RabbitTemplate(connectionFactory);
		rabbitTemplate.setMessageConverter(converter());

		// 消息是否成功发送到Exchange
		rabbitTemplate.setConfirmCallback((correlationData, ack, cause) -> {
			if (ack) {
				log.info("消息成功发送到Exchange");
			} else {
				log.info("消息发送到Exchange失败, {}, cause: {}", correlationData, cause);
			}
		});

		// 触发setReturnCallback回调必须设置mandatory=true, 否则Exchange没有找到Queue就会丢弃掉消息, 而不会触发回调
		rabbitTemplate.setMandatory(true);
		// 消息是否从Exchange路由到Queue, 注意: 这是一个失败回调, 只有消息从Exchange路由到Queue失败才会回调这个方法
		rabbitTemplate.setReturnCallback((message, replyCode, replyText, exchange, routingKey) -> {
			log.info("消息从Exchange路由到Queue失败: exchange: {}, route: {}, replyCode: {}, replyText: {}, message: {}",
					exchange, routingKey, replyCode, replyText, message);
		});

		return rabbitTemplate;
	}

	@Bean
	public Jackson2JsonMessageConverter converter() {
		return new Jackson2JsonMessageConverter();
	}

	@Bean
	public Queue backlogInsertQueue() {
		return new Queue(BacklogConstant.BACKLOG_INSERT, true);
	}

	@Bean
	public Queue backlogUpdateQueue() {
		return new Queue(BacklogConstant.BACKLOG_UPDATE, true);
	}

	@Bean
	public TopicExchange backlogExchange() {
		return new TopicExchange(BacklogConstant.BACKLOG_EXCHANGE, true, false);
	}

	@Bean("topicInsert")
	public Binding topicInsertBinding() {
		return BindingBuilder.bind(backlogInsertQueue()).to(backlogExchange()).with(BacklogConstant.MSG_INSERT);
	}

	@Bean("topicUpdate")
	public Binding topicUpdateBinding() {
		return BindingBuilder.bind(backlogUpdateQueue()).to(backlogExchange()).with(BacklogConstant.MSG_UPDATE);
	}

}
